package com.hexin.android.component.listview

import android.content.Context
import androidx.recyclerview.widget.LinearLayoutManager
import android.util.AttributeSet
import android.view.*
import android.widget.LinearLayout

/**
 * <pre>
 * 这是一个列表控件
 *
 * 主要有以下特点：
 * 1、将表头和列表结合在一起，可以不设置表头
 * 2、支持列表项滑动，前提是Item要实现SlidingItem接口
 *
 * 在使用时，必须设置高度(expectedHeight)
 * </pre>
 */
class ListComponent(context: Context, attrs: AttributeSet) : LinearLayout(context, attrs) {

    val recyclerView = SlidingRecyclerView(context)

    /**
     * ListComponent的高度，必须要设置
     * 标题栏高度 + 列表高度
     */
    var expectedHeight = -1
        set(value) {
            field = value
            adjustRecyclerViewHeight()
        }

    var titleView: View? = null
        set(value) {
            field = value
            addView(field, 0)
            viewTreeObserver.addOnGlobalLayoutListener(object : ViewTreeObserver.OnGlobalLayoutListener {
                override fun onGlobalLayout() {
                    viewTreeObserver.removeOnGlobalLayoutListener(this)
                    adjustRecyclerViewHeight()
                }
            })
        }

    private var mLayoutManager = CustomSmoothTimeLayout(context)

    init {
        orientation = VERTICAL
        mLayoutManager.isItemPrefetchEnabled = false // 不要使用cache，否则会出现左右滑动后列表项错乱的情况
    }

    override fun onFinishInflate() {
        super.onFinishInflate()
        recyclerView.setItemViewCacheSize(0) // 不要使用cache，否则会出现左右滑动后列表项错乱的情况
        recyclerView.layoutManager = mLayoutManager
        recyclerView.overScrollMode = View.OVER_SCROLL_NEVER
        recyclerView.onSlidingListeners.add(object : OnSlidingListener {
            override fun onSliding(scrollX: Int, availableToScrollWidth: Int) {
                val tmpTitleView = titleView
                if (tmpTitleView is SlidingItem) {
                    val slidingView: View? = tmpTitleView.getSlidingView()
                    slidingView?.scrollTo(scrollX, 0)
                    tmpTitleView.onSliding(scrollX)
                }
            }
        })
        addView(recyclerView)
        viewTreeObserver.addOnGlobalLayoutListener(object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                viewTreeObserver.removeOnGlobalLayoutListener(this)
                recyclerView.listNestedScrollView = getListNestedScrollView()
            }
        })
    }

    fun setAdapter(adapter: ListAdapter<*>?) {
        recyclerView.adapter = adapter
    }

    fun scrollToRight(){
        recyclerView.scrollToRight()
    }

    fun scrollToLeft(){
        recyclerView.scrollToLeft()
    }

    private fun adjustRecyclerViewHeight() {
        val lp = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, expectedHeight - (titleView?.height ?: 0))
        recyclerView.layoutParams = lp
    }

    private fun getListNestedScrollView(): ListNestedScrollView? {
        var parent = parent
        while (parent != null) {
            if (parent is ListNestedScrollView) {
                return parent
            }
            parent = parent.parent
        }
        return null
    }
}
